class ParaBlock
  
  # Default contructor
  def initialize(type, content = "", target = nil)
    @orig_type = type
    @type = normalise_type(type)
    
    @content = content
    @target = target
  end
  
  ###
  ### Accessors
  ###
  
  def type=(type)
    @type = normalise_type(type)
  end
  def type
    @type
  end

  def content=(content)
    @content = content
  end
  def content
    @content
  end
  
  def target=(target)
    @target = target
  end
  def target
    @target
  end
  
  ###
  ### Convenience functions
  ###
  
  def <<(value)
    @content << value
  end
  
  def empty?
    return (@content.empty? or (/\S/ !~ @content))
  end
  
  def clear
    @content.clear
  end

  # Returns the length of the type and target, as they were
  # written out _in the original document_. For the current
  # type length, see type_length
  def orig_type_length
    @orig_type.to_s.length + @target.to_s.length + 1
  end
  
  # Returns the length of the type and target, as they would
  # be written out (including the separator), e.g as 
  # 'figure:L3'.length
  def type_length
    @type.to_s.length + @target.to_s.length + 1
  end
  
  # Take the current contents as a block
  # type, and reset the content
  def content_to_type!
    
    unless @content =~ /\s+/ then
      header = @content.split(':')
      
      if header[0].nil? then
        
        # If we can't work out the type, set everything
        # to null
        @type = :none
        @target = nil
      
      else
        
        # Attempt to coerce the content to a type   
        block_type = header[0].to_sym
        unless block_type.nil? then
          @orig_type = block_type
          @type = normalise_type(block_type)
        else
          @orig_type = :none
          @type = :none
        end
        
        @target = header[1]
        
      end
      
    else
      @type = :none
    end
    
    @content.clear
  end
  
  ###
  ### Helper functions
  ###
  
  # Do type mapping if needed, otherwise
  # pass on the contents as the type
  def normalise_type(type)
    case type
      when :e
        return :emph
      when :s
        return :strong
      when :quote
        return :block_quote
      else
        return type
    end
  end
  
  ###
  ### Type conversions
  ###
  
  def self.from_s(string)
    
    unless string =~ /\s+/ then
      header = string.split(':')
      
      if header[0].nil? then
        
        # If we can't work out the type, set everything
        # to null
        type = :none
        target = nil
      
      else
        
        # Attempt to coerce the content to a type   
        block_type = header[0].to_sym
        unless block_type.nil? then
          type = block_type
        else
          type = :none
        end
        
        target = header[1]
      
      end
      
    else
      type = :none
    end
    
    return ParaBlock.new(type, "", target)
  end
  
  def to_s
    @content.to_s
  end
  
  def to_debug_s
    return "block type: #{type}, content: #{content}"
  end
  
  def to_sym
    @content.to_sym
  end
  
  def to_json(*a)
    begin
      require 'json'

      json_hash = {
        "type"    => @name,
        "content" => @content,
        "target"  => @target,
        JSON.create_id => self.class.name
      }

      return json_hash.to_json

    rescue LoadError
      warn "The JSON gem couldn't be loaded, and so the JSON representation could not be generated"
    end
  end

  def self.json_create(json_hash)
    begin
      require 'json'

      block = new(json_hash["type"], json_hash["content"], json_hash["target"])
      return block

    rescue LoadError
      warn "The JSON gem couldn't be loaded, and so the JSON representation could not be generated"
    end
  end
end