# Copyright (c) 2010-2011 David Love
#
# Permission to use, copy, modify, and/or distribute this software for 
# any purpose with or without fee is hereby granted, provided that the 
# above copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES 
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF 
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR 
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

# @author David Love

module WhiteCloth::DataStructures
  
  # Load the UUID class, as extended by the helper library
  require "helpers/uuid"
  
  # Helper class for constructing trees. Takes care of the basic manipulation of the trees, allowing
  # some abstraction in the projections.
  class TreeBuilder
    
    # Default constructor. Creates an internal UUID, which is used to sequence objects
    # added to the {TreeBuilder}.
    def initialize
      
      # Call the parent to create the underlying stack
      super()
      
      # Create our sequence number
      @sequence = UUIDTools::UUID.random_create
      
      # Create the level stack
      @level_stack = Containers::Stack.new
    end
    
  end
  
end